package org.framework.lazy.cloud.network.heartbeat.common.adapter;

import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.flow.HandleChannelFlowAdvanced;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.flow.proxy.ChannelProxyFlow;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.flow.proxy.HandleChannelProxyFlowAdvanced;

import java.util.List;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * 通道流量适配器
 *
 * @see HandleChannelFlowAdvanced
 */
@Slf4j
public class ChannelProxyFlowAdapter {


    ThreadPoolExecutor CHANNEL_FLOW_ADAPTER_EXECUTOR =
            new ThreadPoolExecutor(20, 200, 3L, TimeUnit.MINUTES,
                    new LinkedBlockingDeque<>(500),new ThreadPoolExecutor.AbortPolicy());
    // 线程使用完后使用主线程执行



    protected final List<HandleChannelProxyFlowAdvanced> handleChannelProxyFlowAdvancedList;

    public ChannelProxyFlowAdapter(List<HandleChannelProxyFlowAdvanced> handleChannelProxyFlowAdvancedList) {
        this.handleChannelProxyFlowAdvancedList = handleChannelProxyFlowAdvancedList;
    }


    /**
     * 处理当前数据
     *
     * @param channelProxyFlow 通道数据
     */
    public void handler(Channel channel, ChannelProxyFlow channelProxyFlow) {
        for (HandleChannelProxyFlowAdvanced handleChannelProxyFlowAdvanced : handleChannelProxyFlowAdvancedList) {
            if (handleChannelProxyFlowAdvanced.support(channelProxyFlow)) {
                try {
                    handleChannelProxyFlowAdvanced.handler(channel, channelProxyFlow);
                } catch (Exception e) {
                    log.error("流量统计失败:{}", e.getMessage());
                }
                return;
            }
        }
    }

    /**
     * 异步处理当前数据
     *
     * @param channelProxyFlow 通道数据
     */
    public void asyncHandler(Channel channel, ChannelProxyFlow channelProxyFlow) {
     // TODO 流量并发异常
        CHANNEL_FLOW_ADAPTER_EXECUTOR.submit(() -> handler(channel, channelProxyFlow));
    }

}
