package org.framework.lazy.cloud.network.heartbeat.server.netty.proxy.http.advanced;


import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;
import org.framework.lazy.cloud.network.heartbeat.common.NettyByteBuf;
import org.framework.lazy.cloud.network.heartbeat.common.adapter.ChannelProxyFlowAdapter;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.payload.NettyProxyMsg;
import org.framework.lazy.cloud.network.heartbeat.common.NettyTransferChannelContext;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.payload.NettyChannelContext;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.proxy.http.server.AbstractHandleHttpReportServerProxyClientTransferChannelInitSuccessfulTypeAdvanced;
import org.framework.lazy.cloud.network.heartbeat.common.constant.ProxyMessageType;
import org.framework.lazy.cloud.network.heartbeat.common.enums.ChannelFlowEnum;
import org.framework.lazy.cloud.network.heartbeat.common.enums.ChannelProtocolType;
import org.framework.lazy.cloud.network.heartbeat.common.enums.ChannelProxyType;
import org.framework.lazy.cloud.network.heartbeat.common.utils.ChannelAttributeKeyUtils;
import org.framework.lazy.cloud.network.heartbeat.server.netty.flow.proxy.ServerChannelProxyFlow;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Role;
import org.springframework.stereotype.Component;


@Role(BeanDefinition.ROLE_INFRASTRUCTURE)
@Slf4j
@Component
public class ServerHandleHttpReportServerProxyClientTransferRequestTypeAdvanced
        extends AbstractHandleHttpReportServerProxyClientTransferChannelInitSuccessfulTypeAdvanced<NettyProxyMsg> {

    private final ChannelProxyFlowAdapter channelProxyFlowAdapter;

    public ServerHandleHttpReportServerProxyClientTransferRequestTypeAdvanced(ChannelProxyFlowAdapter channelProxyFlowAdapter) {
        this.channelProxyFlowAdapter = channelProxyFlowAdapter;
    }

    /**
     * 处理当前数据
     *
     * @param nettyChannelContext 当前通道
     * @param nettyProxyMsg   通道数据
     */
    @Override
    public void doHandler(NettyChannelContext nettyChannelContext, NettyProxyMsg nettyProxyMsg) {

        Channel channel = nettyChannelContext.channel();
        // 数据下发
        byte[] msgVisitorId = nettyProxyMsg.getVisitorId();

        String targetIpString = nettyProxyMsg.getTargetIpString();
        String targetPortString = nettyProxyMsg.getTargetPortString();
        Integer targetPort = Integer.parseInt(targetPortString);


        byte[] clientId = nettyProxyMsg.getClientId();


        ChannelAttributeKeyUtils.buildTargetClientId(channel, clientId);
        ChannelAttributeKeyUtils.buildTargetIp(channel, targetIpString);
        ChannelAttributeKeyUtils.buildTargetPort(channel, targetPort);


        // next
        Channel nextChannel = NettyTransferChannelContext.getVisitor(msgVisitorId);
        ChannelAttributeKeyUtils.buildNextChannel(nextChannel, channel);
        ChannelAttributeKeyUtils.buildNextChannel(channel, nextChannel);

        ChannelAttributeKeyUtils.buildClientId(channel, clientId);
        ChannelAttributeKeyUtils.buildVisitorId(channel, msgVisitorId);
        NettyByteBuf nettyByteBufData = ChannelAttributeKeyUtils.getNettyByteBufData(nextChannel);


        log.info("准备发送代理请求数据到客户端");
        NettyProxyMsg clientConnectTagetNettyProxyMsg = new NettyProxyMsg();
        clientConnectTagetNettyProxyMsg.setVisitorId(msgVisitorId);

        clientConnectTagetNettyProxyMsg.setClientTargetIp(targetIpString);
        clientConnectTagetNettyProxyMsg.setClientTargetPort(targetPort);
        clientConnectTagetNettyProxyMsg.setClientId(clientId);
        clientConnectTagetNettyProxyMsg.setData(nettyByteBufData.getData());

        clientConnectTagetNettyProxyMsg.setType(ProxyMessageType.HTTP_DISTRIBUTE_SERVER_PROXY_CLIENT_TRANSFER_REQUEST_);

        channel.writeAndFlush(clientConnectTagetNettyProxyMsg);

        // 记录进口数据
        ServerChannelProxyFlow serverChannelFlow = ServerChannelProxyFlow
                .builder()
                .channelFlowEnum(ChannelFlowEnum.IN_FLOW)
                .port(targetPort)
                .ip(targetIpString)
                .clientId(new String(clientId))
                .channelProxyType(ChannelProxyType.SERVER_PROXY_CLIENT)
                .channelProtocolType(ChannelProtocolType.HTTP)
                .flow(nettyByteBufData.getData().length)
                .build();
        channelProxyFlowAdapter.asyncHandler(channel, serverChannelFlow);


    }

}
