package org.framework.lazy.cloud.network.heartbeat.server.netty.proxy.socks.advanced;


import io.netty.channel.Channel;
import io.netty.util.ReferenceCountUtil;
import lombok.extern.slf4j.Slf4j;
import org.framework.lazy.cloud.network.heartbeat.common.adapter.ChannelProxyFlowAdapter;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.payload.NettyChannelContext;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.payload.NettyProxyMsg;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.proxy.socks.server.AbstractHandleSocksReportServerProxyClientRequestTypeAdvanced;
import org.framework.lazy.cloud.network.heartbeat.common.constant.ProxyMessageType;
import org.framework.lazy.cloud.network.heartbeat.common.enums.ChannelFlowEnum;
import org.framework.lazy.cloud.network.heartbeat.common.enums.ChannelProtocolType;
import org.framework.lazy.cloud.network.heartbeat.common.enums.ChannelProxyType;
import org.framework.lazy.cloud.network.heartbeat.common.utils.ChannelAttributeKeyUtils;
import org.framework.lazy.cloud.network.heartbeat.server.netty.flow.proxy.ServerChannelProxyFlow;
import org.framework.lazy.cloud.network.heartbeat.server.netty.proxy.socks.handler.NettySocksClientProxyServerRealHandler;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Role;
import org.springframework.stereotype.Component;


@Role(BeanDefinition.ROLE_INFRASTRUCTURE)
@Slf4j
@Component
public class ServerHandleSocksReportServerProxyClientRequestTypeAdvanced
        extends AbstractHandleSocksReportServerProxyClientRequestTypeAdvanced<NettyProxyMsg> {
    private final ChannelProxyFlowAdapter channelProxyFlowAdapter;

    public ServerHandleSocksReportServerProxyClientRequestTypeAdvanced(ChannelProxyFlowAdapter channelProxyFlowAdapter) {
        this.channelProxyFlowAdapter = channelProxyFlowAdapter;
    }

    /**
     * 处理当前数据
     *
     * @param nettyChannelContext 当前通道
     * @param nettyProxyMsg       通道数据
     * @see NettySocksClientProxyServerRealHandler
     */
    @Override
    public void doHandler(NettyChannelContext nettyChannelContext, NettyProxyMsg nettyProxyMsg) {

        Channel channel = nettyChannelContext.channel();
        String targetClientId = ChannelAttributeKeyUtils.getTargetClientId(channel);
        String targetIp = ChannelAttributeKeyUtils.getTargetIp(channel);
        Integer targetPort = ChannelAttributeKeyUtils.getTargetPort(channel);


        Channel nextChannel = ChannelAttributeKeyUtils.getNextChannel(channel);

        if (nextChannel.isActive()) {
            // 下发数据到服务端
            NettyProxyMsg nettyProxyMsgRequest = new NettyProxyMsg();
            nettyProxyMsgRequest.setType(ProxyMessageType.SOCKS_DISTRIBUTE_SERVER_PROXY_CLIENT_TRANSFER_REQUEST_);

            nettyProxyMsgRequest.setData(nettyProxyMsg.getData());
            nextChannel.writeAndFlush(nettyProxyMsgRequest);
            // 记录进口数据
            ServerChannelProxyFlow serverChannelFlow = ServerChannelProxyFlow
                    .builder()
                    .channelFlowEnum(ChannelFlowEnum.IN_FLOW)
                    .port(targetPort)
                    .ip(targetIp)
                    .clientId("SERVER")
                    .channelProxyType(ChannelProxyType.SERVER_PROXY_CLIENT)
                    .channelProtocolType(ChannelProtocolType.SOCKS)
                    .flow(nettyProxyMsg.getData().length)
                    .build();
            channelProxyFlowAdapter.asyncHandler(channel, serverChannelFlow);
        } else {
            log.info("释放内存");
            ReferenceCountUtil.release(nettyProxyMsg);
        }
    }

}
