package org.framework.lazy.cloud.network.heartbeat.server.standalone.infrastructure.entity;

import lombok.Data;
import lombok.experimental.Accessors;
import org.framework.lazy.cloud.network.heartbeat.common.enums.ChannelProtocolType;
import org.framework.lazy.cloud.network.heartbeat.common.enums.ChannelProxyType;
import org.wu.framework.lazy.orm.core.stereotype.LazyTableIndex;
import org.wu.framework.core.stereotype.LayerField;
import org.wu.framework.core.stereotype.LayerField.LayerFieldType;
import org.wu.framework.lazy.orm.core.stereotype.LazyTable;
import org.wu.framework.lazy.orm.core.stereotype.LazyTableField;
import org.wu.framework.lazy.orm.core.stereotype.*;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.media.Schema;
import java.lang.String;
import java.time.LocalDateTime;
import org.wu.framework.lazy.orm.core.stereotype.LazyTableFieldId;
import java.lang.Long;
import java.lang.Integer;
import java.lang.Boolean;
/**
 * describe 代理流量 
 *
 * @author Jia wei Wu
 * @date 2025/06/06 11:39 晚上
 * @see org.wu.framework.lazy.orm.core.persistence.reverse.lazy.ddd.DefaultDDDLazyInfrastructureEntity 
 **/
@Data
@Accessors(chain = true)
@LazyTable(tableName = "lazy_proxy_flow",comment = "代理流量")
@Schema(title = "lazy_proxy_flow",description = "代理流量")
public class LazyProxyFlowDO {


    /**
     * 
     * 客户端ID
     */
    @Schema(description ="客户端ID",name ="clientId",example = "")
    @LazyTableFieldUnique(name="client_id",comment="客户端ID",notNull=true,columnType="varchar(50)")
    private String clientId;

    /**
     * 
     * 
     */
    @Schema(description ="",name ="createTime",example = "")
    @LazyTableField(name="create_time",comment="",defaultValue="CURRENT_TIMESTAMP",upsertStrategy = LazyFieldStrategy.NEVER,columnType="datetime",extra="")
    private LocalDateTime createTime;

    /**
     * 
     * 主键
     */
    @Schema(description ="主键",name ="id",example = "")
    @LazyTableFieldId(name = "id", comment = "主键")
    private Long id;

    /**
     * 
     * 当前访客当前进口流量
     */
    @Schema(description ="当前访客当前进口流量",name ="inFlow",example = "")
    @LazyTableField(name="in_flow",comment="当前访客当前进口流量",columnType="int")
    private Integer inFlow;

    /**
     * 
     * 目标ip
     */
    @Schema(description ="目标ip",name ="ip",example = "")
    @LazyTableFieldUnique(name="ip",comment="目标ip",columnType="varchar(25)")
    private String ip;

    /**
     * 
     * 是否删除
     */
    @Schema(description ="是否删除",name ="isDeleted",example = "")
    @LazyTableField(name="is_deleted",comment="是否删除",defaultValue="'0'",upsertStrategy = LazyFieldStrategy.NEVER,columnType="tinyint")
    private Boolean isDeleted;

    /**
     * 
     * 当前访客出口流量
     */
    @Schema(description ="当前访客出口流量",name ="outFlow",example = "")
    @LazyTableField(name="out_flow",comment="当前访客出口流量",columnType="int")
    private Integer outFlow;

    /**
     * 
     * 目标端口
     */
    @Schema(description ="目标端口",name ="port",example = "")
    @LazyTableFieldUnique(name="port",comment="目标端口",columnType="int")
    private Integer port;

    /**
     * 
     * 协议类型
     */
    @Schema(description ="协议类型",name ="protocolType",example = "")
    @LazyTableFieldUnique(name="protocol_type",comment="协议类型",columnType="varchar(25)")
    private ChannelProtocolType protocolType;

    /**
     * 
     * 代理类型
     */
    @Schema(description ="代理类型",name ="proxyType",example = "")
    @LazyTableFieldUnique(name="proxy_type",comment="代理类型",columnType="varchar(25)")
    private ChannelProxyType proxyType;

    /**
     * 
     * 服务端ID
     */
    @Schema(description ="服务端ID",name ="serverId",example = "")
    @LazyTableFieldUnique(name="server_id",comment="服务端ID",notNull=true,columnType="varchar(25)")
    private String serverId;

    /**
     * 
     * 
     */
    @Schema(description ="",name ="updateTime",example = "")
    @LazyTableField(name="update_time",comment="",defaultValue="CURRENT_TIMESTAMP",upsertStrategy = LazyFieldStrategy.NEVER,columnType="datetime",extra=" on update CURRENT_TIMESTAMP")
    private LocalDateTime updateTime;

}